@@ -14,15 +14,24 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2005 Duke University
 
+"""
+Progress display callback classes for the yum command line.
+"""
 
 import rpm
 import os
 import sys
+import logging
 from yum.constants import *
 
-from i18n import _
+from yum.i18n import _
 
 class RPMInstallCallback:
+
+    """
+    Yum command line callback class for callbacks from the RPM library.
+    """
+
     def __init__(self, output=1):
         self.output = output
         self.callbackfilehandles = {}
@@ -32,20 +41,22 @@ class RPMInstallCallback:
         self.total_removed = 0
         self.mark = "#"
         self.marks = 27
-        self.filelog = None
+        self.lastmsg = None
+        self.logger = logging.getLogger('yum.filelogging.RPMInstallCallback')
+        self.filelog = False
 
-        self.myprocess = { TS_UPDATE : 'Updating', 
-                           TS_ERASE: 'Erasing',
-                           TS_INSTALL: 'Installing', 
-                           TS_TRUEINSTALL : 'Installing',
-                           TS_OBSOLETED: 'Obsoleted',
-                           TS_OBSOLETING: 'Installing'}
-        self.mypostprocess = { TS_UPDATE: 'Updated', 
-                               TS_ERASE: 'Erased',
-                               TS_INSTALL: 'Installed', 
-                               TS_TRUEINSTALL: 'Installed', 
-                               TS_OBSOLETED: 'Obsoleted',
-                               TS_OBSOLETING: 'Installed'}
+        self.myprocess = { TS_UPDATE : _('Updating'), 
+                           TS_ERASE: _('Erasing'),
+                           TS_INSTALL: _('Installing'), 
+                           TS_TRUEINSTALL : _('Installing'),
+                           TS_OBSOLETED: _('Obsoleted'),
+                           TS_OBSOLETING: _('Installing')}
+        self.mypostprocess = { TS_UPDATE: _('Updated'), 
+                               TS_ERASE: _('Erased'),
+                               TS_INSTALL: _('Installed'), 
+                               TS_TRUEINSTALL: _('Installed'), 
+                               TS_OBSOLETED: _('Obsoleted'),
+                               TS_OBSOLETING: _('Installed')}
 
         self.tsInfo = None # this needs to be set for anything else to work
 
@@ -105,6 +116,7 @@ class RPMInstallCallback:
             pass
 
         elif what == rpm.RPMCALLBACK_INST_OPEN_FILE:
+            self.lastmsg = None
             hdr = None
             if h is not None:
                 hdr, rpmloc = h
@@ -127,17 +139,19 @@ class RPMInstallCallback:
 
                 # log stuff
                 pkgtup = self._dopkgtup(hdr)
-                txmbr = self.tsInfo.getMembers(pkgtup=pkgtup)[0] # if we have more than one I'll eat my hat
-                try:
-                    process = self.myprocess[txmbr.output_state]
-                    processed = self.mypostprocess[txmbr.output_state]
-                except KeyError, e:
-                    pass
+                
+                txmbrs = self.tsInfo.getMembers(pkgtup=pkgtup)
+                for txmbr in txmbrs:
+                    try:
+                        process = self.myprocess[txmbr.output_state]
+                        processed = self.mypostprocess[txmbr.output_state]
+                    except KeyError:
+                        pass
 
-                if self.filelog:
-                    pkgrep = self._logPkgString(hdr)
-                    msg = '%s: %s' % (processed, pkgrep)
-                    self.filelog(0, msg)
+                    if self.filelog:
+                        pkgrep = self._logPkgString(hdr)
+                        msg = '%s: %s' % (processed, pkgrep)
+                        self.logger.info(msg)
 
 
         elif what == rpm.RPMCALLBACK_INST_PROGRESS:
@@ -151,12 +165,14 @@ class RPMInstallCallback:
                         percent = (bytes*100L)/total
                     if self.output and sys.stdout.isatty():
                         fmt = self._makefmt(percent)
-                        msg = fmt % ('Repackage', h)
+                        msg = fmt % (_('Repackage'), h)
                         if bytes == total:
                             msg = msg + "\n"
-                            
-                        sys.stdout.write(msg)
-                        sys.stdout.flush()
+
+                        if msg != self.lastmsg:
+                            sys.stdout.write(msg)
+                            sys.stdout.flush()
+                            self.lastmsg = msg
                 else:
                     hdr, rpmloc = h
                     if total == 0:
@@ -164,20 +180,24 @@ class RPMInstallCallback:
                     else:
                         percent = (bytes*100L)/total
                     pkgtup = self._dopkgtup(hdr)
-                    txmbr = self.tsInfo.getMembers(pkgtup=pkgtup)[0]
-                    try:
-                        process = self.myprocess[txmbr.output_state]
-                    except KeyError, e:
-                        print "Error: invalid output state: %s for %s" % \
-                           (txmbr.output_state, hdr['name'])
-                    else:
-                        if self.output and sys.stdout.isatty():
-                            fmt = self._makefmt(percent)
-                            msg = fmt % (process, hdr['name'])
-                            sys.stdout.write(msg)
-                            sys.stdout.flush()
-                            if bytes == total:
-                                print " "
+                    
+                    txmbrs = self.tsInfo.getMembers(pkgtup=pkgtup)
+                    for txmbr in txmbrs:
+                        try:
+                            process = self.myprocess[txmbr.output_state]
+                        except KeyError, e:
+                            print _("Error: invalid output state: %s for %s") % \
+                               (txmbr.output_state, hdr['name'])
+                        else:
+                            if self.output and (sys.stdout.isatty() or bytes == total):
+                                fmt = self._makefmt(percent)
+                                msg = fmt % (process, hdr['name'])
+                                if msg != self.lastmsg:
+                                    sys.stdout.write(msg)
+                                    sys.stdout.flush()
+                                    self.lastmsg = msg
+                                if bytes == total:
+                                    print " "
 
 
         elif what == rpm.RPMCALLBACK_UNINST_START:
@@ -188,15 +208,15 @@ class RPMInstallCallback:
 
         elif what == rpm.RPMCALLBACK_UNINST_STOP:
             self.total_removed += 1
-            if h not in self.installed_pkg_names:
+            if self.filelog and h not in self.installed_pkg_names:
                 logmsg = _('Erased: %s' % (h))
-                if self.filelog: self.filelog(0, logmsg)
+                self.logger.info(logmsg)
             
             if self.output and sys.stdout.isatty():
                 if h not in self.installed_pkg_names:
-                    process = "Removing"
+                    process = _("Removing")
                 else:
-                    process = "Cleanup"
+                    process = _("Cleanup")
                 percent = 100
                 fmt = self._makefmt(percent, False)
                 msg = fmt % (process, h)
