@@ -14,32 +14,36 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2004 Duke University
 
+import rpmUtils
 import rpmUtils.miscutils
 import rpmUtils.arch
-import rpmUtils
 
 class Updates:
-    """This class computes and keeps track of updates and obsoletes.
-       initialize, add installed packages, add available packages (both as
-       unique lists of name, epoch, ver, rel, arch tuples), add an optional dict
-       of obsoleting packages with obsoletes and what they obsolete ie:
-        foo, i386, 0, 1.1, 1: bar >= 1.1."""
+    """
+    This class computes and keeps track of updates and obsoletes.
+    initialize, add installed packages, add available packages (both as
+    unique lists of name, epoch, ver, rel, arch tuples), add an optional dict
+    of obsoleting packages with obsoletes and what they obsolete ie::
+        foo, i386, 0, 1.1, 1: bar >= 1.1.
+    """
 
     def __init__(self, instlist, availlist):
-        self.changeTup = [] # storage list tuple of updates or obsoletes
-                            # (oldpkg, newpkg, ['update'|'obsolete'])
 
         self.installed = instlist # list of installed pkgs (n, a, e, v, r)
         self.available = availlist # list of available pkgs (n, a, e, v, r)
+
         self.rawobsoletes = {} # dict of obsoleting package->[what it obsoletes]
+        self._obsoletes_by_name = None
+        self.obsoleted_dict = {}  # obsoleted pkgtup -> [ obsoleting pkgtups ]
+        self.obsoleting_dict = {} # obsoleting pkgtup -> [ obsoleted pkgtups ]
+
         self.exactarch = 1 # don't change archs by default
         self.exactarchlist = ['kernel', 'kernel-smp', 'glibc', 'kernel-hugemem',
                               'kernel-enterprise', 'kernel-bigmem', 'kernel-BOOT']
                               
-        self.myarch = rpmUtils.arch.getCanonArch() # this is for debugging only 
-                                                   # set this if you want to 
-                                                   # test on some other arch
-                                                   # otherwise leave it alone
+        self.myarch = rpmUtils.arch.canonArch # set this if you want to
+                                              # test on some other arch
+                                              # otherwise leave it alone
         
         # make some dicts from installed and available
         self.installdict = self.makeNADict(self.installed, 1)
@@ -47,9 +51,48 @@ class Updates:
 
         # holder for our updates dict
         self.updatesdict = {}
+        self.updating_dict = {}
         #debug, ignore me
         self.debug = 0
 
+    def _delFromDict(self, dict_, keys, value):
+        for key in keys:
+            if not dict_.has_key(key):
+                continue
+            dict_[key] = filter(value.__ne__, dict_[key])
+            if not dict_[key]:
+                del dict_[key]
+
+    def _delFromNADict(self, dict_, pkgtup):
+        (n, a, e, v, r) = pkgtup
+        if dict_.has_key((n, a)):
+            dict_[(n, a)] = filter((e,v,r).__ne__, dict_[(n, a)])
+            if not dict_[(n, a)]:
+                del dict_[(n, a)]
+        if dict_.has_key((n, None)):
+            dict_[(n, None)] = filter((e,v,r).__ne__, dict_[(n, None)])
+            if not dict_[(n, None)]:
+                del dict_[(n, None)]
+
+    def delPackage(self, pkgtup):
+        """remove available pkgtup that is no longer available"""
+        if pkgtup not in self.available:
+            return
+        self.available.remove(pkgtup)
+        self._delFromNADict(self.availdict, pkgtup)
+
+        self._delFromDict(self.updating_dict, self.updatesdict.get(pkgtup, []), pkgtup)
+        self._delFromDict(self.updatesdict, self.updating_dict.get(pkgtup, []), pkgtup)
+
+        if self.rawobsoletes.has_key(pkgtup):
+            if self._obsoletes_by_name:
+                for name, flag, version in self.rawobsoletes[pkgtup]:
+                    self._delFromDict(self._obsoletes_by_name, [name], (flag, version, pkgtup))
+                del self.rawobsoletes[pkgtup]
+
+        self._delFromDict(self.obsoleted_dict, self.obsoleting_dict.get(pkgtup, []), pkgtup)
+        self._delFromDict(self.obsoleting_dict, self.obsoleted_dict.get(pkgtup, []), pkgtup)
+
     def debugprint(self, msg):
         if self.debug:
             print msg
@@ -123,11 +166,45 @@ class Updates:
     def condenseUpdates(self):
         """remove any accidental duplicates in updates"""
         
-        for tup in self.updatesdict.keys():
+        for tup in self.updatesdict:
             if len(self.updatesdict[tup]) > 1:
                 mylist = self.updatesdict[tup]
                 self.updatesdict[tup] = rpmUtils.miscutils.unique(mylist)
+    
+    
+    def checkForObsolete(self, pkglist, newest=1):
+        """accept a list of packages to check to see if anything obsoletes them
+           return an obsoleted_dict in the format of makeObsoletedDict"""
+        if self._obsoletes_by_name is None:
+            self._obsoletes_by_name = {}
+            for pkgtup, obsoletes in self.rawobsoletes.iteritems():
+                for name, flag, version in obsoletes:
+                    self._obsoletes_by_name.setdefault(name, []).append(
+                        (flag, version, pkgtup) )
+
+        obsdict = {} # obseleting package -> [obsoleted package]
+        pkgdict = self.makeNADict(pkglist, 1)
+
+        for pkgtup in pkglist:
+            (name, arch, epoch, ver, rel) = pkgtup
+            for obs_flag, obs_version, obsoleting in self._obsoletes_by_name.get(name, []):
+                if obs_flag in [None, 0] and name == obsoleting[0]: continue
+                if rpmUtils.miscutils.rangeCheck( (name, obs_flag, obs_version), pkgtup):
+                    obsdict.setdefault(obsoleting, []).append(pkgtup)
+
+        obslist = obsdict.keys()
+        if newest:
+            obslist = self._reduceListNewestByNameArch(obslist)
+
+        returndict = {}
+        for new in obslist:
+            for old in obsdict[new]:
+                if not returndict.has_key(old):
+                    returndict[old] = []
+                returndict[old].append(new)
         
+        return returndict
+
     def doObsoletes(self):
         """figures out what things available obsolete things installed, returns
            them in a dict attribute of the class."""
@@ -142,12 +219,20 @@ class Updates:
         # look through all the obsoleting packages look for multiple archs per name
         # if you find it look for the packages they obsolete
         # 
-        for pkgtup in self.rawobsoletes.keys():
+        obs_arches = {}
+        for (n, a, e, v, r) in self.rawobsoletes:
+            if not obs_arches.has_key(n):
+                obs_arches[n] = []
+            obs_arches[n].append(a)
+
+        for pkgtup in self.rawobsoletes:
             (name, arch, epoch, ver, rel) = pkgtup
             for (obs_n, flag, (obs_e, obs_v, obs_r)) in self.rawobsoletes[(pkgtup)]:
-                if flag in [None, 0]: # unversioned obsolete
-                    if self.installdict.has_key((obs_n, None)):
-                        for (rpm_a, rpm_e, rpm_v, rpm_r) in self.installdict[(obs_n, None)]:
+                if self.installdict.has_key((obs_n, None)):
+                    for (rpm_a, rpm_e, rpm_v, rpm_r) in self.installdict[(obs_n, None)]:
+                        if flag in [None, 0] or \
+                                rpmUtils.miscutils.rangeCheck((obs_n, flag, (obs_e, obs_v, obs_r)),
+                                                              (obs_n, rpm_a, rpm_e, rpm_v, rpm_r)):
                             # make sure the obsoleting pkg is not already installed
                             willInstall = 1
                             if self.installdict.has_key((name, None)):
@@ -157,31 +242,12 @@ class Updates:
                                     if self.returnNewest((pkgver, installedver)) == installedver:
                                         willInstall = 0
                                         break
+                            if rpm_a != arch and rpm_a in obs_arches[name]:
+                                willInstall = 0
                             if willInstall:
                                 if not obsdict.has_key(pkgtup):
                                     obsdict[pkgtup] = []
                                 obsdict[pkgtup].append((obs_n, rpm_a, rpm_e, rpm_v, rpm_r))
-
-                else: # versioned obsolete
-                    if self.installdict.has_key((obs_n, None)):
-                        for (rpm_a, rpm_e, rpm_v, rpm_r) in self.installdict[(obs_n, None)]:
-                            if rpmUtils.miscutils.rangeCheck((obs_n, flag, (obs_e, \
-                                                        obs_v, obs_r)), (obs_n,\
-                                                        rpm_a, rpm_e, rpm_v, rpm_r)):
-                                # make sure the obsoleting pkg is not already installed
-                                willInstall = 1
-                                if self.installdict.has_key((name, None)):
-                                    for (ins_a, ins_e, ins_v, ins_r) in self.installdict[(name, None)]:
-                                        pkgver = (epoch, ver, rel)
-                                        installedver = (ins_e, ins_v, ins_r)
-                                        if self.returnNewest((pkgver, installedver)) == installedver:
-                                            willInstall = 0
-                                            break
-                                if willInstall:
-                                    if not obsdict.has_key(pkgtup):
-                                        obsdict[pkgtup] = []
-                                    obsdict[pkgtup].append((obs_n, rpm_a, rpm_e, rpm_v, rpm_r))
-           
         self.obsoletes = obsdict
         self.makeObsoletedDict()
 
@@ -190,11 +256,15 @@ class Updates:
            is to make it easier to look up what package obsoletes what item in 
            the rpmdb"""
         self.obsoleted_dict = {}
-        for new in self.obsoletes.keys():
+        for new in self.obsoletes:
             for old in self.obsoletes[new]:
                 if not self.obsoleted_dict.has_key(old):
                     self.obsoleted_dict[old] = []
                 self.obsoleted_dict[old].append(new)
+        self.obsoleting_dict = {}
+        for obsoleted, obsoletings in self.obsoleted_dict.iteritems():
+            for obsoleting in obsoletings:
+                self.obsoleting_dict.setdefault(obsoleting, []).append(obsoleted)
     
     def doUpdates(self):
         """check for key lists as populated then commit acts of evil to
@@ -241,7 +311,7 @@ class Updates:
 
         # remove the older stuff - if we're doing an update we only want the
         # newest evrs                
-        for (n, a) in newpkgs.keys():
+        for (n, a) in newpkgs:
             if a is None:
                 continue
 
@@ -251,7 +321,7 @@ class Updates:
                     newpkgs[(n, a)].remove((e, v, r))
 
                 
-        for (n, a) in newpkgs.keys():
+        for (n, a) in newpkgs:
             if a is None: # the None archs are only for lookups
                 continue
            
@@ -278,7 +348,7 @@ class Updates:
 
         # ok at this point our newpkgs list should be thinned, we should have only
         # the newest e,v,r's and only archs we can actually use
-        for (n, a) in newpkgs.keys():
+        for (n, a) in newpkgs:
             if a is None: # the None archs are only for lookups
                 continue
     
@@ -338,13 +408,16 @@ class Updates:
         # one for the multiarch itself and one for the compat arch
         # ie: x86_64 and athlon(i686-i386) - we don't want to descend
         # x86_64->i686 
+        # however, we do want to descend x86_64->noarch, sadly.
+        
         archlists = []
         if rpmUtils.arch.isMultiLibArch(arch=self.myarch):
             if rpmUtils.arch.multilibArches.has_key(self.myarch):
                 biarches = [self.myarch]
             else:
                 biarches = [self.myarch, rpmUtils.arch.arches[self.myarch]]
-
+            biarches.append('noarch')
+            
             multicompat = rpmUtils.arch.getMultiArchInfo(self.myarch)[0]
             multiarchlist = rpmUtils.arch.getArchList(multicompat)
             archlists = [ biarches, multiarchlist ]
@@ -366,7 +439,6 @@ class Updates:
                 tmplist = []
                 for (a, e, v, r) in newpkgs[(n, None)]:
                     tmplist.append((n, a, e, v, r))                        
-                
                 highestavailablepkgs = self.returnHighestVerFromAllArchsByName(n,
                                          thisarchlist, tmplist)
 
@@ -393,29 +465,26 @@ class Updates:
                     # is more than one arch updating with the highest ver
                     instarchs = []
                     availarchs = []
-                    for (n,a) in hipdict.keys():
+                    for (n,a) in hipdict:
                         instarchs.append(a)
-                    for (n,a) in hapdict.keys():
+                    for (n,a) in hapdict:
                         availarchs.append(a)
                     
                     rpm_a = rpmUtils.arch.getBestArchFromList(instarchs, myarch=self.myarch)
                     a = rpmUtils.arch.getBestArchFromList(availarchs, myarch=self.myarch)
-
+                    
                     if rpm_a is None or a is None:
                         continue
                         
                     (rpm_e, rpm_v, rpm_r) = hipdict[(n, rpm_a)][0] # there can be just one
                     (e, v, r) = hapdict[(n, a)][0] # just one, I'm sure, I swear!
-
                     rc = rpmUtils.miscutils.compareEVR((e, v, r), (rpm_e, rpm_v, rpm_r))
-
                     if rc > 0:
                         # this is definitely an update - put it in the dict
                         if not updatedict.has_key((n, rpm_a, rpm_e, rpm_v, rpm_r)):
                             updatedict[(n, rpm_a, rpm_e, rpm_v, rpm_r)] = []
                         updatedict[(n, rpm_a, rpm_e, rpm_v, rpm_r)].append((n, a, e, v, r))
                    
-                   
         self.updatesdict = updatedict                    
         self.makeUpdatingDict()
         
@@ -424,7 +493,7 @@ class Updates:
            is to make it easier to look up what package  will be updating what
            in the rpmdb"""
         self.updating_dict = {}
-        for old in self.updatesdict.keys():
+        for old in self.updatesdict:
             for new in self.updatesdict[old]:
                 if not self.updating_dict.has_key(new):
                     self.updating_dict[new] = []
@@ -454,8 +523,7 @@ class Updates:
         """returns updates for packages in a list of tuples of:
           (updating naevr, installed naevr)"""
         returnlist = []
-        for oldtup in self.updatesdict.keys():
-            (old_n, old_a, old_e, old_v, old_r) = oldtup
+        for oldtup in self.updatesdict:
             for newtup in self.updatesdict[oldtup]:
                 returnlist.append((newtup, oldtup))
         
@@ -481,7 +549,7 @@ class Updates:
         """returns updating packages in a list of (naevr) tuples"""
         returnlist = []
 
-        for oldtup in self.updatesdict.keys():
+        for oldtup in self.updatesdict:
             for newtup in self.updatesdict[oldtup]:
                 returnlist.append(newtup)
         
@@ -541,7 +609,7 @@ class Updates:
     def getObsoletedList(self, newest=0, name=None):
         """returns a list of pkgtuples obsoleting the package in name"""
         returnlist = []
-        for new in self.obsoletes.keys():
+        for new in self.obsoletes:
             for obstup in self.obsoletes[new]:
                 (n, a, e, v, r) = obstup
                 if n == name:
