@@ -15,28 +15,25 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2002 Duke University 
 
+"""
+Configuration parser and default values for yum.
+"""
+
 import os
 import warnings
 import rpm
 import copy
 import urlparse
-import sys
-from parser import IncludingConfigParser, IncludedDirConfigParser
-from ConfigParser import NoSectionError, NoOptionError
+from parser import ConfigPreProcessor
+try:
+    from iniparse.compat import NoSectionError, NoOptionError, ConfigParser
+    from iniparse.compat import ParsingError
+except ImportError:
+    from ConfigParser import NoSectionError, NoOptionError, ConfigParser
+    from ConfigParser import ParsingError
 import rpmUtils.transaction
 import rpmUtils.arch
 import Errors
-from repos import Repository
-
-class OptionData(object):
-    '''
-    Simple class to track state for a single option instance.
-    '''
-    def __init__(self, name, initial_value):
-        self.name = name
-        self.value = initial_value
-        self.parser = None
-        self.section = None
 
 class Option(object):
     '''
@@ -68,11 +65,8 @@ class Option(object):
         '''
         if obj is None:
             return self
-        optdata = getattr(obj, self._attrname, None)
-        if optdata == None:
-            return None
-        else:
-            return optdata.value
+
+        return getattr(obj, self._attrname, None)
 
     def __set__(self, obj, value):
         '''Called when the option is set (via the descriptor protocol). 
@@ -81,23 +75,15 @@ class Option(object):
         @param value: The value to set the option to.
         @return: Nothing.
         '''
-        optdata = getattr(obj, self._attrname)
-       
         # Only try to parse if its a string
         if isinstance(value, basestring):
             try:
                 value = self.parse(value)
             except ValueError, e:
                 # Add the field name onto the error
-                raise ValueError('Error parsing %r: %s' % (optdata.name,
-                    str(e)))
+                raise ValueError('Error parsing %r: %s' % (value, str(e)))
 
-        optdata.value = value
-
-        # Write string value back to parser instance if possible
-        if optdata.parser != None:
-            strvalue = self.tostring(value)
-            optdata.parser.set(optdata.section, optdata.name, strvalue)
+        setattr(obj, self._attrname, value)
 
     def setup(self, obj, name):
         '''Initialise the option for a config instance. 
@@ -106,19 +92,7 @@ class Option(object):
         @param obj: BaseConfig (or subclass) instance.
         @param name: Name of the option.
         '''
-        setattr(obj, self._attrname, OptionData(name, self.default))
-
-    def setparser(self, obj, parser, section):
-        '''Set the configuration parser for this option. This is required so
-        that options can be written back to a configuration file.
-
-        @param obj: BaseConfig (or subclass) instance.
-        @param parser: ConfigParser (or subclass) where the option is read from.
-        @param section: config file section where the option is from.
-        '''
-        optdata = getattr(obj, self._attrname)
-        optdata.parser = parser
-        optdata.section = section
+        setattr(obj, self._attrname, copy.copy(self.default))
 
     def clone(self):
         '''Return a safe copy of this Option instance
@@ -164,6 +138,10 @@ def Inherit(option_obj):
 
 class ListOption(Option):
 
+    """
+    An option containing a list of strings.
+    """
+
     def __init__(self, default=None):
         if default is None:
             default = []
@@ -240,13 +218,94 @@ class UrlListOption(ListOption):
 
 
 class IntOption(Option):
+
+    """
+    An option representing an integer value.
+    """
+
+    def __init__(self, default=None, range_min=None, range_max=None):
+        super(IntOption, self).__init__(default)
+        self._range_min = range_min
+        self._range_max = range_max
+        
     def parse(self, s):
         try:
-            return int(s)
+            val = int(s)
         except (ValueError, TypeError), e:
             raise ValueError('invalid integer value')
+        if self._range_max is not None and val > self._range_max:
+            raise ValueError('out of range integer value')
+        if self._range_min is not None and val < self._range_min:
+            raise ValueError('out of range integer value')
+        return val
+
+class PositiveIntOption(IntOption):
+
+    """
+    An option representing a positive integer value, where 0 can have a special
+    represention.
+    """
+
+    def __init__(self, default=None, range_min=0, range_max=None,
+                 names_of_0=None):
+        super(PositiveIntOption, self).__init__(default, range_min, range_max)
+        self._names0 = names_of_0
+
+    def parse(self, s):
+        if s in self._names0:
+            return 0
+        return super(PositiveIntOption, self).parse(s)
+
+class SecondsOption(Option):
+
+    """
+    An option representing an integer value of seconds, or a human readable
+    variation specifying days, hours, minutes or seconds until something
+    happens. Works like BytesOption.
+    Note that due to historical president -1 means "never", so this accepts
+    that and allows the word never too.
+
+    Valid inputs: 100, 1.5m, 90s, 1.2d, 1d, 0xF, 0.1, -1, never
+    Invalid inputs: -10, -0.1, 45.6Z, 1d6h, 1day, 1y
+
+    Return value will always be an integer
+    """
+    MULTS = {'d': 60 * 60 * 24, 'h' : 60 * 60, 'm' : 60, 's': 1}
+
+    def parse(self, s):
+        if len(s) < 1:
+            raise ValueError("no value specified")
+
+        if s == "-1" or s == "never": # Special cache timeout, meaning never
+            return -1
+        if s[-1].isalpha():
+            n = s[:-1]
+            unit = s[-1].lower()
+            mult = self.MULTS.get(unit, None)
+            if not mult:
+                raise ValueError("unknown unit '%s'" % unit)
+        else:
+            n = s
+            mult = 1
+
+        try:
+            n = float(n)
+        except (ValueError, TypeError), e:
+            raise ValueError('invalid value')
+
+        if n < 0:
+            raise ValueError("seconds value may not be negative")
+
+        return int(n * mult)
 
 class BoolOption(Option):
+
+    """
+    An option representing a boolean value.
+
+    The value can be one of 0, 1, yes, no, true, or false.
+    """
+
     def parse(self, s):
         s = s.lower()
         if s in ('0', 'no', 'false'):
@@ -258,9 +317,9 @@ class BoolOption(Option):
 
     def tostring(self, value):
         if value:
-            return "yes"
+            return "1"
         else:
-            return "no"
+            return "0"
 
 class FloatOption(Option):
     def parse(self, s):
@@ -283,6 +342,11 @@ class SelectionOption(Option):
 
 class BytesOption(Option):
 
+    """
+    An option representing a value in bytes.
+
+    The value may be given in bytes, kilobytes, megabytes, or gigabytes.
+    """
     # Multipliers for unit symbols
     MULTS = {
         'k': 1024,
@@ -389,8 +453,8 @@ class BaseConfig(object):
         @param parent: Optional parent BaseConfig (or subclass) instance to use
             when doing option value inheritance.
         '''
+        self.cfg = parser
         self._section = section
-        self.cfg = parser           # Keep a reference to the parser
 
         for name in self.iterkeys():
             option = self.optionobj(name)
@@ -402,21 +466,14 @@ class BaseConfig(object):
                 if parent and option.inherit:
                     value = getattr(parent, name)
                
-            option.setparser(self, parser, section)
             if value is not None:
                 setattr(self, name, value)
 
     def optionobj(cls, name):
         '''Return the Option instance for the given name
         '''
-        # Look for Option instances in this class and base classes so that
-        # option inheritance works
-        for klass in (cls,) + cls.__bases__:
-            obj = klass.__dict__.get(name, None)
-            if obj is not None:
-                break
-
-        if obj is not None and isinstance(obj, Option):
+        obj = getattr(cls, name, None)
+        if isinstance(obj, Option):
             return obj
         else:
             raise KeyError
@@ -448,6 +505,31 @@ class BaseConfig(object):
             if self.isoption(name):
                 yield (name, getattr(self, name))
 
+    def write(self, fileobj, section=None, always=()):
+        '''Write out the configuration to a file-like object
+
+        @param fileobj: File-like object to write to
+        @param section: Section name to use. If not-specified the section name
+            used during parsing will be used.
+        @param always: A sequence of option names to always write out.
+            Options not listed here will only be written out if they are at
+            non-default values. Set to None to dump out all options.
+        '''
+        # Write section heading
+        if section is None:
+            if self._section is None:
+                raise ValueError("not populated, don't know section")
+            section = self._section
+
+        # Updated the ConfigParser with the changed values    
+        cfgOptions = self.cfg.options(section)
+        for name,value in self.iteritems():
+            option = self.optionobj(name)
+            if always is None or name in always or option.default != value or name in cfgOptions :
+                self.cfg.set(section,name, option.tostring(value))
+        # write the updated ConfigParser to the fileobj.
+        self.cfg.write(fileobj)
+
     def getConfigOption(self, option, default=None):
         warnings.warn('getConfigOption() will go away in a future version of Yum.\n'
                 'Please access option values as attributes or using getattr().',
@@ -465,28 +547,33 @@ class BaseConfig(object):
         else:
             raise Errors.ConfigError, 'No such option %s' % option
 
-class EarlyConf(BaseConfig):
+class StartupConf(BaseConfig):
     '''
     Configuration option definitions for yum.conf's [main] section that are
-    required before the other [main] options can be parsed (mainly due to
-    variable substitution).
+    required early in the initialisation process or before the other [main]
+    options can be parsed. 
     '''
-    distroverpkg = Option('fedora-release')
-    installroot = Option('/')
+    debuglevel = IntOption(2, 0, 10)
+    errorlevel = IntOption(2, 0, 10)
 
-class YumConf(EarlyConf):
+    distroverpkg = Option('redhat-release')
+    installroot = Option('/')
+    config_file_path = Option('/etc/yum/yum.conf')
+    plugins = BoolOption(False)
+    pluginpath = ListOption(['/usr/share/yum-plugins', '/usr/lib/yum-plugins'])
+    pluginconfpath = ListOption(['/etc/yum/pluginconf.d'])
+
+class YumConf(StartupConf):
     '''
     Configuration option definitions for yum.conf\'s [main] section.
 
-    Note: inherits options from EarlyConf too.
+    Note: see also options inherited from StartupConf
     '''
-
-    debuglevel = IntOption(2)
-    errorlevel = IntOption(2)
-    retries = IntOption(10)
-    recent = IntOption(7)
+    retries = PositiveIntOption(10, names_of_0=["<forever>"])
+    recent = IntOption(7, range_min=0)
 
     cachedir = Option('/var/cache/yum')
+    persistdir = Option('/var/lib/yum')
     keepcache = BoolOption(True)
     logfile = Option('/var/log/yum.log')
     reposdir = ListOption(['/etc/yum/repos.d', '/etc/yum.repos.d'])
@@ -496,19 +583,23 @@ class YumConf(EarlyConf):
     commands = ListOption()
     exclude = ListOption()
     failovermethod = Option('roundrobin')
-    yumversion = Option('unversioned')
     proxy = UrlOption(schemes=('http', 'ftp', 'https'), allow_none=True)
     proxy_username = Option()
     proxy_password = Option()
-    pluginpath = ListOption(['/usr/lib/yum-plugins'])
     installonlypkgs = ListOption(['kernel', 'kernel-bigmem',
             'kernel-enterprise','kernel-smp', 'kernel-modules', 'kernel-debug',
-            'kernel-unsupported', 'kernel-source', 'kernel-devel'])
+            'kernel-unsupported', 'kernel-source', 'kernel-devel', 'kernel-PAE',
+            'kernel-PAE-debug'])
+    # NOTE: If you set this to 2, then because it keeps the current kernel it
+    # means if you ever install an "old" kernel it'll get rid of the newest one
+    # so you probably want to use 3 as a minimum ... if you turn it on.
+    installonly_limit = PositiveIntOption(0, range_min=2,
+                                          names_of_0=["0", "<off>"])
     kernelpkgnames = ListOption(['kernel','kernel-smp', 'kernel-enterprise',
-            'kernel-bigmem', 'kernel-BOOT'])
+            'kernel-bigmem', 'kernel-BOOT', 'kernel-PAE', 'kernel-PAE-debug'])
     exactarchlist = ListOption(['kernel', 'kernel-smp', 'glibc',
             'kernel-hugemem', 'kernel-enterprise', 'kernel-bigmem',
-            'kernel-devel'])
+            'kernel-devel', 'kernel-PAE', 'kernel-PAE-debug'])
     tsflags = ListOption()
 
     assumeyes = BoolOption(False)
@@ -522,17 +613,36 @@ class YumConf(EarlyConf):
     obsoletes = BoolOption(False)
     showdupesfromrepos = BoolOption(False)
     enabled = BoolOption(True)
-    plugins = BoolOption(False)
     enablegroups = BoolOption(True)
     enable_group_conditionals = BoolOption(True)
-
-    timeout = FloatOption(30.0)
+    group_package_types = ListOption(['mandatory', 'default'])
+    
+    timeout = FloatOption(30.0) # FIXME: Should use variation of SecondsOption
 
     bandwidth = BytesOption(0)
     throttle = ThrottleOption(0)
 
     http_caching = SelectionOption('all', ('none', 'packages', 'all'))
-    metadata_expire = IntOption(1800)   # time in seconds
+    #  Time in seconds (1.5h), yum-updatesd runs once per. hour by default
+    # this time means we don't do interactive network checks/updates if
+    # yum-updatesd is running.
+    metadata_expire = SecondsOption(60 * 90)
+    mirrorlist_expire = SecondsOption(86400) # time in seconds (1 day)
+    rpm_check_debug = BoolOption(True)
+    disable_excludes = ListOption()    
+    skip_broken = BoolOption(False)
+    #  Note that "instant" is the old behaviour, but group:primary is very
+    # similar but better :).
+    mdpolicy = SelectionOption('group:primary',
+                               ('instant', 'group:all', 'group:main',
+                                'group:small', 'group:primary'))
+    multilib_policy = SelectionOption('all',('best', 'all'))
+                 # all == install any/all arches you can
+                 # best == use the 'best  arch' for the system
+                 
+
+    
+    _reposlist = []
 
 class RepoConf(BaseConfig):
     '''
@@ -542,6 +652,7 @@ class RepoConf(BaseConfig):
     enabled = Inherit(YumConf.enabled)
     baseurl = UrlListOption()
     mirrorlist = UrlOption()
+    mediaid = Option()
     gpgkey = UrlListOption()
     exclude = ListOption() 
     includepkgs = ListOption() 
@@ -561,98 +672,91 @@ class RepoConf(BaseConfig):
     timeout = Inherit(YumConf.timeout)
     http_caching = Inherit(YumConf.http_caching)
     metadata_expire = Inherit(YumConf.metadata_expire)
+    mirrorlist_expire = Inherit(YumConf.mirrorlist_expire)
+    mdpolicy = Inherit(YumConf.mdpolicy)
+    cost = IntOption(1000)
+    
+def readStartupConfig(configfile, root):
+    '''
+    Parse Yum's main configuration file and return a StartupConf instance.
+    
+    This is required in order to access configuration settings required as Yum
+    starts up.
 
-def readMainConfig(configfile, root):
-    '''Parse Yum's main configuration file
-
-    @param configfile: Path to the configuration file to parse (typically
-        '/etc/yum.conf').
+    @param configfile: The path to yum.conf.
     @param root: The base path to use for installation (typically '/')
-    @return: Populated YumConf instance.
+    @return: A StartupConf instance.
+
+    May raise Errors.ConfigError if a problem is detected with while parsing.
     '''
 
-    # Read up config variables that are needed early to calculate substitution
-    # variables
-    EarlyConf.installroot.default = root
-    earlyconf = EarlyConf()
-    confparser = IncludingConfigParser()
-    if not os.path.exists(configfile):
-        raise Errors.ConfigError, 'No such config file %s' % configfile
+    StartupConf.installroot.default = root
+    startupconf = StartupConf()
+    startupconf.config_file_path = configfile
+    parser = ConfigParser()
+    confpp_obj = ConfigPreProcessor(configfile)
+    try:
+        parser.readfp(confpp_obj)
+    except ParsingError, e:
+        raise Errors.ConfigError("Parsing file failed: %s" % e)
+    startupconf.populate(parser, 'main')
 
-    confparser.read(configfile)
-    earlyconf.populate(confparser, 'main')
+    # Check that plugin paths are all absolute
+    for path in startupconf.pluginpath:
+        if not path[0] == '/':
+            raise Errors.ConfigError("All plugin search paths must be absolute")
 
+    # Stuff this here to avoid later re-parsing
+    startupconf._parser = parser
+
+    return startupconf
+
+def readMainConfig(startupconf):
+    '''
+    Parse Yum's main configuration file
+
+    @param startupconf: StartupConf instance as returned by readStartupConfig()
+    @return: Populated YumConf instance.
+    '''
+    
     # Set up substitution vars
-    vars = _getEnvVar()
-    vars['basearch'] = rpmUtils.arch.getBaseArch()          # FIXME make this configurable??
-    vars['arch'] = rpmUtils.arch.getCanonArch()             # FIXME make this configurable??
-    vars['releasever'] = _getsysver(earlyconf.installroot, earlyconf.distroverpkg)
+    yumvars = _getEnvVar()
+    yumvars['basearch'] = rpmUtils.arch.getBaseArch()          # FIXME make this configurable??
+    yumvars['arch'] = rpmUtils.arch.getCanonArch()             # FIXME make this configurable??
+    yumvars['releasever'] = _getsysver(startupconf.installroot, startupconf.distroverpkg)
 
     # Read [main] section
     yumconf = YumConf()
-    confparser = IncludingConfigParser(vars=vars)
-    confparser.read(configfile)
-    yumconf.populate(confparser, 'main')
+    yumconf.populate(startupconf._parser, 'main')
 
     # Apply the installroot to directory options
-    for option in ('cachedir', 'logfile'):
+    for option in ('cachedir', 'logfile', 'persistdir'):
         path = getattr(yumconf, option)
         setattr(yumconf, option, yumconf.installroot + path)
     
-    # Check that plugin paths are all absolute
-    for path in yumconf.pluginpath:
-        if not path.startswith('/'):
-            raise Errors.ConfigError("All plugin search paths must be absolute")
-
     # Add in some extra attributes which aren't actually configuration values 
-    yumconf.yumvar = vars
+    yumconf.yumvar = yumvars
     yumconf.uid = 0
     yumconf.cache = 0
     yumconf.progess_obj = None
-
+    
+    # items related to the originating config file
+    yumconf.config_file_path = startupconf.config_file_path
+    yumconf.config_file_age = os.stat(startupconf.config_file_path)[8]
+    
+    # propagate the debuglevel and errorlevel values:
+    yumconf.debuglevel = startupconf.debuglevel
+    yumconf.errorlevel = startupconf.errorlevel
+    
     return yumconf
 
-def readRepoConfig(parser, section, mainconf):
-    '''Parse an INI file section for a repository.
-
-    @param parser: ConfParser or similar to read INI file values from.
-    @param section: INI file section to read.
-    @param mainconf: ConfParser or similar for yum.conf.
-    @return: Repository instance.
-    '''
-
-    conf = RepoConf()
-    conf.populate(parser, section, mainconf)
-
-    # Ensure that the repo name is set
-    if not conf.name:
-        conf.name = section
-        print >> sys.stderr, \
-            'Repository %r is missing name in configuration, using id' % section
-
-    thisrepo = Repository(section)
-
-    # Transfer attributes across
-    #TODO: merge RepoConf and Repository 
-    for k, v in conf.iteritems():
-        if v or not hasattr(thisrepo, k):
-            thisrepo.set(k, v)
-
-    # Set attributes not from the config file
-    thisrepo.basecachedir = mainconf.cachedir
-    thisrepo.yumvar.update(mainconf.yumvar)
-    thisrepo.cfg = parser
-
-    return thisrepo
-
-def getOption(conf, section, name, default, option):
+def getOption(conf, section, name, option):
     '''Convenience function to retrieve a parsed and converted value from a
     ConfigParser.
 
     @param conf: ConfigParser instance or similar
     @param section: Section name
     @param name: Option name
-    @param default: Value to use if option is missing
     @param option: Option instance to use for conversion.
     @return: The parsed value or default if value was not present.
 
@@ -661,7 +765,7 @@ def getOption(conf, section, name, default, option):
     try: 
         val = conf.get(section, name)
     except (NoSectionError, NoOptionError):
-        return default
+        return option.default
     return option.parse(val)
 
 def _getEnvVar():
@@ -690,7 +794,7 @@ def _getsysver(installroot, distroverpkg):
     # we're going to take the first one - if there is more than one of these
     # then the user needs a beating
     if idx.count() == 0:
-        releasever = 'Null'
+        releasever = '$releasever'
     else:
         hdr = idx.next()
         releasever = hdr['version']
@@ -699,9 +803,8 @@ def _getsysver(installroot, distroverpkg):
     del ts
     return releasever
 
-
 #def main():
-#    mainconf = readMainConfig('/etc/yum.conf', '/') 
+#    mainconf = readMainConfig('/etc/yum/yum.conf', '/')
 #    repoconf = readRepoConfig(mainconf.cfg, 'core', mainconf)
 #
 #    print `repoconf.name`
