@@ -16,57 +16,35 @@
 import os
 import glob
 import imp
+import warnings
 import atexit
+import gettext
+import logging
+import logginglevels
 from constants import *
-import ConfigParser
+try:
+    import iniparse.compat as ConfigParser
+except ImportError:
+    import ConfigParser
 import config 
 import Errors
+from parser import ConfigPreProcessor
 
-# XXX: break the API for how plugins define config file options
-#   - they should just be able to manipulate the YumConf and RepoConf classes
-#     directly, adding Option instances as required
-#   - cleaner, more flexible and 
-#   - update PLUGINS document
-#   - will take care of the following existing TODOs:
-# TODO: detect conflicts between builtin yum options and registered plugin
-# options (will require refactoring of config.py)
-# TODO: plugins should be able to specify convertor functions for config vars
+from textwrap import fill
+from i18n import _
 
-# TODO: should plugin searchpath be affected by installroot option?
-
-# TODO: cleaner method to query installed packages rather than exposing RpmDB
-# (Panu?)
-
-# TODO: consistent case of YumPlugins methods
 
+# TODO: expose rpm package sack objects to plugins (once finished)
+# TODO: allow plugins to use the existing config stuff to define options for
+# their own configuration files (would replace confString() etc).
 # TODO: expose progress bar interface
-
-# TODO: allow plugins to define new repository types
-
-# TODO: check for *_hook methods that aren't supported
-
 # TODO "log" slot? To allow plugins to do customised logging/history (say to a
 # SQL db)
-
-# TODO: multiversion plugin support
-
-# TODO: config vars marked as PLUG_OPT_WHERE_ALL should inherit defaults from
-#   the [main] setting if the user doesn't specify them
-
+# TODO: consistent case of YumPlugins methods
 # TODO: allow plugins to extend shell commands
-
-# TODO: allow plugins to extend commands (on the command line)
-
+# TODO: allow plugins to define new repository types
 # TODO: More developer docs:  use epydoc as API begins to stablise
 
-# TODO: test the API by implementing some crack from bugzilla
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=181 
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=270
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=310
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=431
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=88 (?)
-#         - http://devel.linux.duke.edu/bugzilla/show_bug.cgi?id=396 (DONE)
-
 
 # The API_VERSION constant defines the current plugin API version. It is used
 # to decided whether or not plugins can be loaded. It is compared against the
@@ -82,22 +60,30 @@ import Errors
 # API, the major version number must be incremented and the minor version number
 # reset to 0. If a change is made that doesn't break backwards compatibility,
 # then the minor number must be incremented.
-API_VERSION = '2.2'
+API_VERSION = '2.6'
+
+class DeprecatedInt(int):
+    '''
+    A simple int subclass that used to check when a deprecated constant is used.
+    '''
 
 # Plugin types
 TYPE_CORE = 0
-TYPE_INTERFACE = 1
-ALL_TYPES = (TYPE_CORE, TYPE_INTERFACE)
+TYPE_INTERACTIVE = 1
+TYPE_INTERFACE = DeprecatedInt(1)
+ALL_TYPES = (TYPE_CORE, TYPE_INTERACTIVE)
 
 # Mapping of slots to conduit classes
 SLOT_TO_CONDUIT = {
     'config': 'ConfigPluginConduit',
+    'postconfig': 'PostConfigPluginConduit',
     'init': 'InitPluginConduit',
     'predownload': 'DownloadPluginConduit',
     'postdownload': 'DownloadPluginConduit',
     'prereposetup': 'PreRepoSetupPluginConduit',
     'postreposetup': 'PostRepoSetupPluginConduit',
     'close': 'PluginConduit',
+    'clean': 'PluginConduit',
     'pretrans': 'MainPluginConduit',
     'posttrans': 'MainPluginConduit',
     'exclude': 'MainPluginConduit',
@@ -111,13 +97,22 @@ SLOTS = SLOT_TO_CONDUIT.keys()
 class PluginYumExit(Exception):
     '''Used by plugins to signal that yum should stop
     '''
-
+    def __init__(self, value="", translation_domain=""):
+        self.value = value
+        self.translation_domain = translation_domain
+    def __str__(self):
+        if self.translation_domain:
+            return gettext.dgettext(self.translation_domain, self.value)
+        else:
+            return self.value
+    
 class YumPlugins:
     '''
     Manager class for Yum plugins.
     '''
 
-    def __init__(self, base, searchpath, optparser=None, types=None):
+    def __init__(self, base, searchpath, optparser=None, types=None, 
+            pluginconfpath=None,disabled=None):
         '''Initialise the instance.
 
         @param base: The
@@ -127,18 +122,31 @@ class YumPlugins:
         @param types: A sequence specifying the types of plugins to load.
             This should be sequnce containing one or more of the TYPE_...
             constants. If None (the default), all plugins will be loaded.
+        @param pluginconfpath: A list of paths to look for plugin configuration
+            files. Defaults to "/etc/yum/pluginconf.d".
         '''
+        if not pluginconfpath:
+            pluginconfpath = ['/etc/yum/pluginconf.d']
 
         self.searchpath = searchpath
+        self.pluginconfpath = pluginconfpath
         self.base = base
         self.optparser = optparser
         self.cmdline = (None, None)
-        if not types:
+        self.verbose_logger = logging.getLogger("yum.verbose.YumPlugins")
+        self.disabledPlugins = disabled
+        if types is None:
             types = ALL_TYPES
+        if not isinstance(types, (list, tuple)):
+            types = (types,)
+
+        if id(TYPE_INTERFACE) in [id(t) for t in types]:
+            self.verbose_logger.log(logginglevels.INFO_2,
+                    'Deprecated constant TYPE_INTERFACE during plugin '
+                    'initialization.\nPlease use TYPE_INTERACTIVE instead.')
 
         self._importplugins(types)
 
-        self.opts = {}
         self.cmdlines = {}
 
         # Call close handlers when yum exit's
@@ -157,8 +165,8 @@ class YumPlugins:
         conduitcls = eval(conduitcls)       # Convert name to class object
 
         for modname, func in self._pluginfuncs[slotname]:
-            self.base.log(3, 'Running "%s" handler for "%s" plugin' % (
-                slotname, modname))
+            self.verbose_logger.debug('Running "%s" handler for "%s" plugin', slotname,
+                modname)
     
             _, conf = self._plugins[modname]
             func(conduitcls(self, self.base, conf, **kwargs))
@@ -179,6 +187,15 @@ class YumPlugins:
                 continue
             for modulefile in glob.glob('%s/*.py' % dir):
                 self._loadplugin(modulefile, types)
+            plugins = sorted(self._plugins)
+
+            # Mostly copied from YumOutput._outKeyValFill()
+            key = _("Loaded plugins: ")
+            val = ", ".join(plugins)
+            nxt = ' ' * (len(key) - 2) + ': '
+            self.verbose_logger.log(logginglevels.INFO_2,
+                                    fill(val, width=80, initial_indent=key,
+                                         subsequent_indent=nxt))
 
     def _loadplugin(self, modulefile, types):
         '''Attempt to import a plugin module and register the hook methods it
@@ -188,22 +205,25 @@ class YumPlugins:
         modname = modname.split('.py')[0]
 
         conf = self._getpluginconf(modname)
-        if not conf or not config.getOption(conf, 'main', 'enabled', False,
-                config.BoolOption()):
-            self.base.log(3, '"%s" plugin is disabled' % modname)
+        if not conf or not config.getOption(conf, 'main', 'enabled', 
+                config.BoolOption(False)):
+            self.verbose_logger.debug(_('"%s" plugin is disabled'), modname)
             return
 
         fp, pathname, description = imp.find_module(modname, [dir])
-        module = imp.load_module(modname, fp, pathname, description)
+        try:
+            module = imp.load_module(modname, fp, pathname, description)
+        finally:
+            fp.close()
 
         # Check API version required by the plugin
         if not hasattr(module, 'requires_api_version'):
              raise Errors.ConfigError(
-                'Plugin "%s" doesn\'t specify required API version' % modname
+                _('Plugin "%s" doesn\'t specify required API version') % modname
                 )
         if not apiverok(API_VERSION, module.requires_api_version):
             raise Errors.ConfigError(
-                'Plugin "%s" requires API %s. Supported API is %s.' % (
+                _('Plugin "%s" requires API %s. Supported API is %s.') % (
                     modname,
                     module.requires_api_version,
                     API_VERSION,
@@ -217,17 +237,28 @@ class YumPlugins:
         if len(plugintypes) < 1:
             return
         for plugintype in plugintypes:
+            if id(plugintype) == id(TYPE_INTERFACE):
+                self.verbose_logger.log(logginglevels.INFO_2,
+                        'Plugin "%s" uses deprecated constant '
+                        'TYPE_INTERFACE.\nPlease use TYPE_INTERACTIVE '
+                        'instead.', modname)
+
             if plugintype not in types:
                 return
+        # Check if this plugin has been temporary disabled
+        if self.disabledPlugins:
+            if modname in self.disabledPlugins:
+                return
 
-        self.base.log(2, 'Loading "%s" plugin' % modname)
+        self.verbose_logger.log(logginglevels.DEBUG_3, _('Loading "%s" plugin'),
+                                modname)
 
         # Store the plugin module and its configuration file
         if not self._plugins.has_key(modname):
             self._plugins[modname] = (module, conf)
         else:
-            raise Errors.ConfigError('Two or more plugins with the name "%s" ' \
-                    'exist in the plugin search path' % modname)
+            raise Errors.ConfigError(_('Two or more plugins with the name "%s" ' \
+                    'exist in the plugin search path') % modname)
         
         for slot in SLOTS:
             funcname = slot+'_hook'
@@ -241,67 +272,26 @@ class YumPlugins:
         IncludingConfigParser instance representing it. Returns None if there
         was an error reading or parsing the configuration file.
         '''
-        #XXX: should this use installroot?
-        conffilename = os.path.join('/etc/yum/pluginconf.d', modname+'.conf')
-
+        for dir in self.pluginconfpath:
+            conffilename = os.path.join(dir, modname + ".conf")
+            if os.access(conffilename, os.R_OK):
+                # Found configuration file
+                break
+            self.verbose_logger.log(logginglevels.INFO_2, _("Configuration file %s not found") % conffilename)
+        else: # for
+            # Configuration files for the plugin not found
+            self.verbose_logger.log(logginglevels.INFO_2, _("Unable to find configuration file for plugin %s")
+                % modname)
+            return None
+        parser = ConfigParser.ConfigParser()
+        confpp_obj = ConfigPreProcessor(conffilename)
         try:
-            parser = config.IncludingConfigParser()
-            parser.read(conffilename)
-        except ConfigParser.Error, e:
+            parser.readfp(confpp_obj)
+        except ParsingError, e:
             raise Errors.ConfigError("Couldn't parse %s: %s" % (conffilename,
                 str(e)))
-        except IOError, e:
-            self.base.log(2, str(e))
-            return None
-
         return parser
 
-    def registeropt(self, name, valuetype, where, default):
-        '''Called from plugins to register a new config file option.
-
-        @param name: Name of the new option.
-        @param valuetype: Option type (PLUG_OPT_BOOL, PLUG_OPT_STRING ...)
-        @param where: Where the option should be available in the config file.
-            (PLUG_OPT_WHERE_MAIN, PLUG_OPT_WHERE_REPO, ...)
-        @param default: Default value for the option if not set by the user.
-        '''
-        if self.opts.has_key(name):
-            raise Errors.ConfigError('Plugin option conflict: ' \
-                    'an option named "%s" has already been registered' % name
-                    )
-        self.opts[name] = (valuetype, where, default)
-
-    def parseopts(self, conf, repos):
-        '''Parse any configuration options registered by plugins
-
-        @param conf: the yumconf instance holding Yum's global options
-        @param repos: a list of all repository objects
-        '''
-        #XXX: with the new config stuff this is an ugly hack!
-        # See first TODO at top of this file
-
-        type2opt =  {
-            PLUG_OPT_STRING: config.Option(),
-            PLUG_OPT_INT: config.IntOption(),
-            PLUG_OPT_BOOL: config.BoolOption(),
-            PLUG_OPT_FLOAT: config.FloatOption(),
-            }
-
-        # Process [main] options first
-        for name, (vtype, where, default) in self.opts.iteritems(): 
-            if where in (PLUG_OPT_WHERE_MAIN, PLUG_OPT_WHERE_ALL):
-                val = config.getOption(conf.cfg, 'main', name, default,
-                        type2opt[vtype])
-                setattr(conf, name, val)
-
-        # Process repository level options
-        for repo in repos:
-            for name, (vtype, where, default) in self.opts.iteritems(): 
-                if where in (PLUG_OPT_WHERE_REPO, PLUG_OPT_WHERE_ALL):
-                    val = config.getOption(conf.cfg, repo.id, name, default,
-                            type2opt[vtype])
-                    repo.set(name, val)
-
     def setCmdLine(self, opts, commands):
         '''Set the parsed command line options so that plugins can access them
         '''
@@ -325,11 +315,16 @@ class PluginConduit:
         self._base = base
         self._conf = conf
 
+        self.logger = logging.getLogger("yum.plugin")
+        self.verbose_logger = logging.getLogger("yum.verbose.plugin")
+
     def info(self, level, msg):
-        self._base.log(level, msg)
+        converted_level = logginglevels.logLevelFromDebugLevel(level)
+        self.verbose_logger.log(converted_level, msg)
 
     def error(self, level, msg):
-        self._base.errorlog(level, msg)
+        converted_level = logginglevels.logLevelFromErrorLevel(level)
+        self.logger.log(converted_level, msg)
 
     def promptYN(self, msg):
         self.info(2, msg)
@@ -367,8 +362,7 @@ class PluginConduit:
         @param default: Value to read if option is missing.
         @return: String option value read, or default if option was missing.
         '''
-        return config.getOption(self._conf, section, opt, default,
-                config.Option())
+        return config.getOption(self._conf, section, opt, config.Option(default))
 
     def confInt(self, section, opt, default=None):
         '''Read an integer value from the plugin's own configuration file
@@ -379,8 +373,7 @@ class PluginConduit:
         @return: Integer option value read, or default if option was missing or
             could not be parsed.
         '''
-        return config.getOption(self._conf, section, opt, default,
-                config.IntOption())
+        return config.getOption(self._conf, section, opt, config.IntOption(default))
 
     def confFloat(self, section, opt, default=None):
         '''Read a float value from the plugin's own configuration file
@@ -391,8 +384,7 @@ class PluginConduit:
         @return: Float option value read, or default if option was missing or
             could not be parsed.
         '''
-        return config.getOption(self._conf, section, opt, default,
-                config.FloatOption())
+        return config.getOption(self._conf, section, opt, config.FloatOption(default))
 
     def confBool(self, section, opt, default=None):
         '''Read a boolean value from the plugin's own configuration file
@@ -403,18 +395,51 @@ class PluginConduit:
         @return: Boolean option value read, or default if option was missing or
             could not be parsed.
         '''
-        return config.getOption(self._conf, section, opt, default,
-                config.BoolOption())
+        return config.getOption(self._conf, section, opt, config.BoolOption(default))
 
 class ConfigPluginConduit(PluginConduit):
 
-    def registerOpt(self, *args, **kwargs):
+    def registerOpt(self, name, valuetype, where, default):
         '''Register a yum configuration file option.
 
-        Arguments are as for YumPlugins.registeropt().
+        @param name: Name of the new option.
+        @param valuetype: Option type (PLUG_OPT_BOOL, PLUG_OPT_STRING ...)
+        @param where: Where the option should be available in the config file.
+            (PLUG_OPT_WHERE_MAIN, PLUG_OPT_WHERE_REPO, ...)
+        @param default: Default value for the option if not set by the user.
         '''
-        self._parent.registeropt(*args, **kwargs)
+        warnings.warn('registerOpt() will go away in a future version of Yum.\n'
+                'Please manipulate config.YumConf and config.RepoConf directly.',
+                DeprecationWarning)
 
+        type2opt =  {
+            PLUG_OPT_STRING: config.Option,
+            PLUG_OPT_INT: config.IntOption,
+            PLUG_OPT_BOOL: config.BoolOption,
+            PLUG_OPT_FLOAT: config.FloatOption,
+            }
+
+        if where == PLUG_OPT_WHERE_MAIN:
+            setattr(config.YumConf, name, type2opt[valuetype](default))
+
+        elif where == PLUG_OPT_WHERE_REPO:
+            setattr(config.RepoConf, name, type2opt[valuetype](default))
+
+        elif where == PLUG_OPT_WHERE_ALL:
+            option = type2opt[valuetype](default)
+            setattr(config.YumConf, name, option)
+            setattr(config.RepoConf, name, config.Inherit(option))
+
+    def registerCommand(self, command):
+        if hasattr(self._base, 'registerCommand'):
+            self._base.registerCommand(command)
+        else:
+            raise Errors.ConfigError(_('registration of commands not supported'))
+
+class PostConfigPluginConduit(ConfigPluginConduit):
+
+    def getConf(self):
+        return self._base.conf
 
 class InitPluginConduit(PluginConduit):
 
@@ -443,8 +468,6 @@ class PreRepoSetupPluginConduit(InitPluginConduit):
 
         @return: rpmUtils.RpmDBHolder instance
         '''
-        self._base.doTsSetup()
-        self._base.doRpmDBSetup()
         return self._base.rpmdb
 
 class PostRepoSetupPluginConduit(PreRepoSetupPluginConduit):
@@ -454,7 +477,6 @@ class PostRepoSetupPluginConduit(PreRepoSetupPluginConduit):
 
         @return: yum.comps.Comps instance
         '''
-        self._base.doGroupSetup()
         return self._base.comps
 
 class DownloadPluginConduit(PostRepoSetupPluginConduit):
@@ -500,7 +522,7 @@ class MainPluginConduit(PostRepoSetupPluginConduit):
         return self._base.getPackageObject(nevra)
 
     def delPackage(self, po):
-        self._base.pkgSack.delPackage(po)
+        po.repo.sack.delPackage(po)
 
     def getTsInfo(self):
         return self._base.tsInfo
@@ -511,6 +533,7 @@ class DepsolvePluginConduit(MainPluginConduit):
         self.resultcode = rescode
         self.resultstring = restring
 
+
 def parsever(apiver):
     maj, min = apiver.split('.')
     return int(maj), int(min)
